@@ -41,10 +41,21 @@
 #include "device-private.h"
 #include "context-private.h"
 
-
-void
-device_init (dc_device_t *device, dc_context_t *context, const dc_device_vtable_t *vtable)
+dc_device_t *
+dc_device_allocate (dc_context_t *context, const dc_device_vtable_t *vtable)
 {
+	dc_device_t *device = NULL;
+
+	assert(vtable != NULL);
+	assert(vtable->size >= sizeof(dc_device_t));
+
+	// Allocate memory.
+	device = (dc_device_t *) malloc (vtable->size);
+	if (device == NULL) {
+		ERROR (context, "Failed to allocate memory.");
+		return device;
+	}
+
 	device->vtable = vtable;
 
 	device->context = context;
@@ -58,6 +69,14 @@ device_init (dc_device_t *device, dc_context_t *context, const dc_device_vtable_
 
 	memset (&device->devinfo, 0, sizeof (device->devinfo));
 	memset (&device->clock, 0, sizeof (device->clock));
+
+	return device;
+}
+
+void
+dc_device_deallocate (dc_device_t *device)
+{
+	free (device);
 }
 
 dc_status_t
@@ -164,7 +183,7 @@ dc_device_open (dc_device_t **out, dc_context_t *context, dc_descriptor_t *descr
 		rc = citizen_aqualand_device_open (&device, context, name);
 		break;
 	case DC_FAMILY_DIVESYSTEM_IDIVE:
-		rc = divesystem_idive_device_open (&device, context, name);
+		rc = divesystem_idive_device_open2 (&device, context, name, dc_descriptor_get_model (descriptor));
 		break;
 	default:
 		return DC_STATUS_INVALIDARGS;
@@ -355,17 +374,22 @@ dc_device_foreach (dc_device_t *device, dc_dive_callback_t callback, void *userd
 dc_status_t
 dc_device_close (dc_device_t *device)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
+
 	if (device == NULL)
 		return DC_STATUS_SUCCESS;
 
-	if (device->vtable->close == NULL)
-		return DC_STATUS_UNSUPPORTED;
-
 	// Disable the cancellation callback.
 	device->cancel_callback = NULL;
 	device->cancel_userdata = NULL;
 
-	return device->vtable->close (device);
+	if (device->vtable->close) {
+		status = device->vtable->close (device);
+	}
+
+	dc_device_deallocate (device);
+
+	return status;
 }
 
 
