@@ -52,6 +52,7 @@ static dc_status_t shearwater_petrel_device_foreach (dc_device_t *abstract, dc_d
 static dc_status_t shearwater_petrel_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t shearwater_petrel_device_vtable = {
+	sizeof(shearwater_petrel_device_t),
 	DC_FAMILY_SHEARWATER_PETREL,
 	shearwater_petrel_device_set_fingerprint, /* set_fingerprint */
 	NULL, /* read */
@@ -80,34 +81,35 @@ str2num (unsigned char data[], unsigned int size, unsigned int offset)
 dc_status_t
 shearwater_petrel_device_open (dc_device_t **out, dc_context_t *context, const char *name)
 {
-	dc_status_t rc = DC_STATUS_SUCCESS;
+	dc_status_t status = DC_STATUS_SUCCESS;
+	shearwater_petrel_device_t *device = NULL;
 
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	shearwater_petrel_device_t *device = (shearwater_petrel_device_t *) malloc (sizeof (shearwater_petrel_device_t));
+	device = (shearwater_petrel_device_t *) dc_device_allocate (context, &shearwater_petrel_device_vtable);
 	if (device == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Initialize the base class.
-	device_init (&device->base.base, context, &shearwater_petrel_device_vtable);
-
 	// Set the default values.
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 
 	// Open the device.
-	rc = shearwater_common_open (&device->base, context, name);
-	if (rc != DC_STATUS_SUCCESS) {
-		free (device);
-		return rc;
+	status = shearwater_common_open (&device->base, context, name);
+	if (status != DC_STATUS_SUCCESS) {
+		goto error_free;
 	}
 
 	*out = (dc_device_t *) device;
 
 	return DC_STATUS_SUCCESS;
+
+error_free:
+	dc_device_deallocate ((dc_device_t *) device);
+	return status;
 }
 
 
@@ -126,9 +128,6 @@ shearwater_petrel_device_custom_open (dc_device_t **out, dc_context_t *context,
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Initialize the base class.
-	device_init (&device->base.base, context, &shearwater_petrel_device_vtable);
-
 	// Set the default values.
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 
@@ -148,7 +147,6 @@ shearwater_petrel_device_custom_open (dc_device_t **out, dc_context_t *context,
 static dc_status_t
 shearwater_petrel_device_close (dc_device_t *abstract)
 {
-	dc_status_t rc = DC_STATUS_SUCCESS;
 	shearwater_common_device_t *device = (shearwater_common_device_t *) abstract;
 
 	// Shutdown the device.
@@ -156,12 +154,7 @@ shearwater_petrel_device_close (dc_device_t *abstract)
 	shearwater_common_transfer (device, request, sizeof (request), NULL, 0, NULL);
 
 	// Close the device.
-	rc = shearwater_common_close (device);
-
-	// Free memory.
-	free (device);
-
-	return rc;
+	return shearwater_common_close (device);
 }
 
 
