@@ -53,6 +53,7 @@ static dc_status_t shearwater_predator_device_foreach (dc_device_t *abstract, dc
 static dc_status_t shearwater_predator_device_close (dc_device_t *abstract);
 
 static const dc_device_vtable_t shearwater_predator_device_vtable = {
+	sizeof(shearwater_predator_device_t),
 	DC_FAMILY_SHEARWATER_PREDATOR,
 	shearwater_predator_device_set_fingerprint, /* set_fingerprint */
 	NULL, /* read */
@@ -66,34 +67,35 @@ static const dc_device_vtable_t shearwater_predator_device_vtable = {
 dc_status_t
 shearwater_predator_device_open (dc_device_t **out, dc_context_t *context, const char *name)
 {
-	dc_status_t rc = DC_STATUS_SUCCESS;
+	dc_status_t status = DC_STATUS_SUCCESS;
+	shearwater_predator_device_t *device = NULL;
 
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	shearwater_predator_device_t *device = (shearwater_predator_device_t *) malloc (sizeof (shearwater_predator_device_t));
+	device = (shearwater_predator_device_t *) dc_device_allocate (context, &shearwater_predator_device_vtable);
 	if (device == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Initialize the base class.
-	device_init (&device->base.base, context, &shearwater_predator_device_vtable);
-
 	// Set the default values.
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 
 	// Open the device.
-	rc = shearwater_common_open (&device->base, context, name);
-	if (rc != DC_STATUS_SUCCESS) {
-		free (device);
-		return rc;
+	status = shearwater_common_open (&device->base, context, name);
+	if (status != DC_STATUS_SUCCESS) {
+		goto error_free;
 	}
 
 	*out = (dc_device_t *) device;
 
 	return DC_STATUS_SUCCESS;
+
+error_free:
+	dc_device_deallocate ((dc_device_t *) device);
+	return status;
 }
 
 
@@ -112,9 +114,6 @@ shearwater_predator_device_custom_open (dc_device_t **out, dc_context_t *context
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Initialize the base class.
-	device_init (&device->base.base, context, &shearwater_predator_device_vtable);
-
 	// Set the default values.
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 
@@ -134,16 +133,9 @@ shearwater_predator_device_custom_open (dc_device_t **out, dc_context_t *context
 static dc_status_t
 shearwater_predator_device_close (dc_device_t *abstract)
 {
-	dc_status_t rc = DC_STATUS_SUCCESS;
 	shearwater_common_device_t *device = (shearwater_common_device_t *) abstract;
 
-	// Close the device.
-	rc = shearwater_common_close (device);
-
-	// Free memory.
-	free (device);
-
-	return rc;
+	return shearwater_common_close (device);
 }
 
 
