@@ -55,6 +55,7 @@ static dc_status_t suunto_d9_device_close (dc_device_t *abstract);
 
 static const suunto_common2_device_vtable_t suunto_d9_device_vtable = {
 	{
+		sizeof(suunto_d9_device_t),
 		DC_FAMILY_SUUNTO_D9,
 		suunto_common2_device_set_fingerprint, /* set_fingerprint */
 		suunto_common2_device_read, /* read */
@@ -129,18 +130,21 @@ suunto_d9_device_autodetect (suunto_d9_device_t *device, unsigned int model)
 dc_status_t
 suunto_d9_device_open (dc_device_t **out, dc_context_t *context, const char *name, unsigned int model)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
+	suunto_d9_device_t *device = NULL;
+
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
-	suunto_d9_device_t *device = (suunto_d9_device_t *) malloc (sizeof (suunto_d9_device_t));
+	device = (suunto_d9_device_t *) dc_device_allocate (context, &suunto_d9_device_vtable.base);
 	if (device == NULL) {
 		ERROR (context, "Failed to allocate memory.");
 		return DC_STATUS_NOMEMORY;
 	}
 
 	// Initialize the base class.
-	suunto_common2_device_init (&device->base, context, &suunto_d9_device_vtable);
+	suunto_common2_device_init (&device->base);
 
 	// Set the default values.
 	device->port = NULL;
@@ -149,33 +153,30 @@ suunto_d9_device_open (dc_device_t **out, dc_context_t *context, const char *nam
 	int rc = serial_open (&device->port, context, name);
 	if (rc == -1) {
 		ERROR (context, "Failed to open the serial port.");
-		free (device);
-		return DC_STATUS_IO;
+		status = DC_STATUS_IO;
+		goto error_free;
 	}
 
 	// Set the serial communication protocol (9600 8N1).
 	rc = serial_configure (device->port, 9600, 8, SERIAL_PARITY_NONE, 1, SERIAL_FLOWCONTROL_NONE);
 	if (rc == -1) {
 		ERROR (context, "Failed to set the terminal attributes.");
-		serial_close (device->port);
-		free (device);
-		return DC_STATUS_IO;
+		status = DC_STATUS_IO;
+		goto error_close;
 	}
 
 	// Set the timeout for receiving data (3000 ms).
 	if (serial_set_timeout (device->port, 3000) == -1) {
 		ERROR (context, "Failed to set the timeout.");
-		serial_close (device->port);
-		free (device);
-		return DC_STATUS_IO;
+		status = DC_STATUS_IO;
+		goto error_close;
 	}
 
 	// Set the DTR line (power supply for the interface).
 	if (serial_set_dtr (device->port, 1) == -1) {
 		ERROR (context, "Failed to set the DTR line.");
-		serial_close (device->port);
-		free (device);
-		return DC_STATUS_IO;
+		status = DC_STATUS_IO;
+		goto error_close;
 	}
 
 	// Give the interface 100 ms to settle and draw power up.
@@ -185,12 +186,10 @@ suunto_d9_device_open (dc_device_t **out, dc_context_t *context, const char *nam
 	serial_flush (device->port, SERIAL_QUEUE_BOTH);
 
 	// Try to autodetect the protocol variant.
-	dc_status_t status = suunto_d9_device_autodetect (device, model);
+	status = suunto_d9_device_autodetect (device, model);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (context, "Failed to identify the protocol variant.");
-		serial_close (device->port);
-		free (device);
-		return status;
+		goto error_close;
 	}
 
 	// Override the base class values.
@@ -205,24 +204,27 @@ suunto_d9_device_open (dc_device_t **out, dc_context_t *context, const char *nam
 	*out = (dc_device_t*) device;
 
 	return DC_STATUS_SUCCESS;
+
+error_close:
+	serial_close (device->port);
+error_free:
+	dc_device_deallocate ((dc_device_t *) device);
+	return status;
 }
 
 
 static dc_status_t
 suunto_d9_device_close (dc_device_t *abstract)
 {
+	dc_status_t status = DC_STATUS_SUCCESS;
 	suunto_d9_device_t *device = (suunto_d9_device_t*) abstract;
 
 	// Close the device.
 	if (serial_close (device->port) == -1) {
-		free (device);
-		return DC_STATUS_IO;
+		dc_status_set_error(&status, DC_STATUS_IO);
 	}
 
-	// Free memory.
-	free (device);
-
-	return DC_STATUS_SUCCESS;
+	return status;
 }
 
 
